package com.fireworksdk.bridge.components.storyblock

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.View.LAYOUT_DIRECTION_LOCALE
import android.view.ViewGroup
import android.widget.FrameLayout
import com.firework.storyblock.FeedLoadListener
import com.firework.storyblock.FwStoryBlockView
import com.fireworksdk.bridge.components.base.FWBaseFragment
import com.fireworksdk.bridge.databinding.FwBridgeStoryBlockBinding
import com.fireworksdk.bridge.models.FWVideoFeedPropsModel
import com.fireworksdk.bridge.models.FWVideoFeedPropsModelDeserializer
import com.fireworksdk.bridge.models.FWVideoFeedPropsModelSerializer
import com.fireworksdk.bridge.utils.FWCommonUtil
import com.fireworksdk.bridge.utils.FWConfigUtil
import com.fireworksdk.bridge.utils.FWLanguageUtil
import org.json.JSONObject

class StoryBlockFragment : FWBaseFragment() {

  private var _binding: FwBridgeStoryBlockBinding? = null
  private val binding: FwBridgeStoryBlockBinding get() = _binding!!

  private var feedLoadListener: FeedLoadListener? = null
  private var fullScreenStateChangeListener: FwStoryBlockView.OnFullscreenStateChangeListener? = null
  private var videoFeedPropsModel: FWVideoFeedPropsModel? = null

  private var isStoryBlockInitializer = false
  private var storyBlockLayoutParams: FrameLayout.LayoutParams? = null

  private var fwStoryBlockView: FwStoryBlockView? = null

  override fun onCreateView(
    inflater: LayoutInflater,
    container: ViewGroup?,
    savedInstanceState: Bundle?,
  ): View {
    _binding = FwBridgeStoryBlockBinding.inflate(inflater)

    val videoFeedPropsModelJsonString  = arguments?.getString(VIDEO_FEED_PROPS_MODEL)
    videoFeedPropsModelJsonString?.let {
      val videoFeedPropsModelJsonObject = JSONObject(it)
      val model = FWVideoFeedPropsModelDeserializer.deserialize(videoFeedPropsModelJsonObject)
      videoFeedPropsModel = model
    }

    val right = videoFeedPropsModel?.marginRight
    if (right != null && right > 0) {
      val layoutParams = FrameLayout.LayoutParams(
        ViewGroup.LayoutParams.MATCH_PARENT,
        ViewGroup.LayoutParams.MATCH_PARENT
      )
      layoutParams.setMargins(0, 0, FWCommonUtil.dpToPx(right.toFloat(), requireContext()), 0)
      binding.storyBlockContainer.layoutParams = layoutParams
    }

    FWLanguageUtil.getInstance(requireContext()).updateBaseContextLocale(requireContext())
    binding.storyBlock.layoutDirection = LAYOUT_DIRECTION_LOCALE

    fwStoryBlockView = binding.storyBlock
    return binding.root
  }

  override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
    super.onViewCreated(view, savedInstanceState)
    initStoryBlock()
  }


  override fun onDestroyView() {
    _binding = null
    super.onDestroyView()
  }

  private fun initStoryBlock() {
    val viewOptionsBuilder = FWConfigUtil.generateViewOptionsBuilder(context, videoFeedPropsModel)
    fwStoryBlockView?.init(
      childFragmentManager,
      lifecycle,
      viewOptionsBuilder.build(),
      binding.storyBlockContainer,
    )
    isStoryBlockInitializer = true

    if (storyBlockLayoutParams != null) {
      binding.storyBlockContainer.layoutParams = storyBlockLayoutParams
      binding.root.requestLayout()
    }
    // must call it after init()
    fwStoryBlockView?.setFeedLoadListener(feedLoadListener)
    fwStoryBlockView?.setOnFullscreenStateChangeListener(fullScreenStateChangeListener)
  }

  // may be called before onCreateView
  fun setStoryBlockLayoutParams(l: FrameLayout.LayoutParams?) {
    l?: return
    storyBlockLayoutParams = l
    if (isStoryBlockInitializer) {
      binding.storyBlockContainer.layoutParams = l
      binding.root.requestLayout()
    }
  }

  // may be called before onCreateView
  fun setFeedLoadListener(feedLoadListener: FeedLoadListener?) {
    this.feedLoadListener = feedLoadListener
    if (isStoryBlockInitializer) {
      fwStoryBlockView?.setFeedLoadListener(feedLoadListener)
    }
  }

  // may be called before onCreateView
  fun setOnFullScreenStateChangedListener(listener: FwStoryBlockView.OnFullscreenStateChangeListener?) {
    this.fullScreenStateChangeListener = listener
    if (isStoryBlockInitializer) {
      fwStoryBlockView?.setOnFullscreenStateChangeListener(fullScreenStateChangeListener)
    }
  }

  fun play() {
    fwStoryBlockView?.play()
  }

  fun pause() {
    fwStoryBlockView?.pause()
  }

  fun toggleFullScreen() {
    fwStoryBlockView?.toggleFullscreen()
  }

  fun isFullScreen(): Boolean {
    return fwStoryBlockView?.isFullscreen() ?: false
  }

  fun getStoryBlockView(): FwStoryBlockView? {
    return fwStoryBlockView
  }

  companion object {
    private const val VIDEO_FEED_PROPS_MODEL = "videoFeedPropsModel"

    fun newInstance(
      videoFeedPropsModel: FWVideoFeedPropsModel?,
    ): StoryBlockFragment {
      return StoryBlockFragment().apply {
        arguments = Bundle().apply {
          videoFeedPropsModel?.let {
            putString(VIDEO_FEED_PROPS_MODEL, FWVideoFeedPropsModelSerializer.serialize(it).toString())
          }
        }
      }
    }
  }
}
