package com.fireworksdk.bridge.components.videofeed

import android.content.Context
import android.util.AttributeSet
import android.view.ViewGroup
import android.widget.FrameLayout
import com.firework.videofeed.FwVideoFeedView
import com.fireworksdk.bridge.models.*
import com.fireworksdk.bridge.models.enums.FWVideoFeedMode
import com.fireworksdk.bridge.models.enums.FWVideoFeedSource
import com.fireworksdk.bridge.utils.FWConfigUtil
import com.fireworksdk.bridge.utils.FWLanguageUtil


class FWVideoFeed(
  context: Context,
  attrs: AttributeSet?
) : FrameLayout(context, attrs) {

  constructor(context: Context) : this(context, null)

  val videoFeedView = FwVideoFeedView(context)
  var videoFeedPropsModel: FWVideoFeedPropsModel = FWVideoFeedPropsModel()

  init {
    FWLanguageUtil.getInstance(context).updateBaseContextLocale(context)
    videoFeedView.layoutDirection = LAYOUT_DIRECTION_LOCALE
    addView(videoFeedView, LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT))
  }

  fun setVideoFeedLayoutParams(l: LayoutParams?) {
    l ?: return
    videoFeedView.layoutParams = l
    requestLayout()
  }

  fun afterUpdateTransaction() {
    val viewOptionsBuilder = FWConfigUtil.generateViewOptionsBuilder(context, videoFeedPropsModel)
    videoFeedView.init(viewOptionsBuilder.build())
  }

  fun setProps(props: FWVideoFeedPropsModel?) {
    props ?: return
    videoFeedPropsModel = props
    afterUpdateTransaction()
  }

  fun setSourceProps(source: String?) {
    videoFeedPropsModel = videoFeedPropsModel.copy(
      source = if (!source.isNullOrBlank()) FWVideoFeedSource.deserialize(source) else null
    )
  }

  fun setChannelProps(channel: String?) {
    videoFeedPropsModel = videoFeedPropsModel.copy(
      channel = channel
    )
  }

  fun setPlaylistProps(playlist: String?) {
    videoFeedPropsModel = videoFeedPropsModel.copy(
      playlist = playlist
    )
  }

  fun setPlaylistGroupProps(playlistGroup: String?) {
    videoFeedPropsModel = videoFeedPropsModel.copy(
      playlistGroup = playlistGroup
    )
  }

  fun setHashtagFilterExpressionProps(hashtagFilterExpression: String?) {
    videoFeedPropsModel = videoFeedPropsModel.copy(
      hashtagFilterExpression = hashtagFilterExpression
    )
  }

  fun setModeProps(mode: String?) {
    videoFeedPropsModel = videoFeedPropsModel.copy(
      mode = if (!mode.isNullOrBlank()) FWVideoFeedMode.deserialize(mode) else null
    )
  }

  fun setEnablePictureInPicture(enablePictureInPicture: Boolean?) {
    videoFeedPropsModel = videoFeedPropsModel.copy(
      enablePictureInPicture = enablePictureInPicture
    )
  }

  fun setDynamicContentParametersProps(dynamicContentParameters: HashMap<String, List<String>>?) {
    videoFeedPropsModel = videoFeedPropsModel.copy(
      dynamicContentParameters = dynamicContentParameters
    )
  }

  fun setVideoFeedConfigProps(config: FWVideoFeedConfigModel?) {
    videoFeedPropsModel = videoFeedPropsModel.copy(
      videoFeedConfiguration = config
    )
  }

  fun setVideoPlayerConfigProps(config: FWVideoPlayerConfigModel?) {
    videoFeedPropsModel = videoFeedPropsModel.copy(
      videoPlayerConfiguration = config
    )
  }

  fun refresh() {
    videoFeedView.refresh()
  }

}
