package com.fireworksdk.bridge.flutter.handler

import android.app.Activity
import com.firework.sdk.FireworkSdk
import com.fireworksdk.bridge.FWInitializationProvider
import com.fireworksdk.bridge.flutter.models.FWNavigatorModuleCallMethod
import com.fireworksdk.bridge.utils.FWGlobalDataUtil
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel

class FWNavigatorModuleHandler : MethodChannel.MethodCallHandler {

  override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
    when (call.method) {
      FWNavigatorModuleCallMethod.PushNativeContainer.rawValue -> {
        result.success(true)
      }

      FWNavigatorModuleCallMethod.PopNativeContainer.rawValue -> {
        val activity = FWInitializationProvider.INSTANCE.resumedActivity
        if (activity == null) {
          result.success(false)
          return
        }
        if (isTaskRoot(activity)) {
          if (FWGlobalDataUtil.storyBlockFragment?.isFullScreen() == true) {
            FWGlobalDataUtil.storyBlockFragment?.toggleFullScreen()
            result.success(true)
            return
          }

          result.success(false)
          return
        }

        activity.finish()
        result.success(true)
      }

      FWNavigatorModuleCallMethod.CanPopNativeContainer.rawValue -> {
        val activity = FWInitializationProvider.INSTANCE.resumedActivity
        if (activity == null) {
          result.success(false)
          return
        }
        result.success(!isTaskRoot(activity))
      }

      FWNavigatorModuleCallMethod.StartFloatingPlayer.rawValue -> {
        FireworkSdk.enterPip(onSuccess = {
          result.success(true)
        }, onError = {
          result.success(false)
        })
      }

      FWNavigatorModuleCallMethod.StopFloatingPlayer.rawValue -> {
        FireworkSdk.closePip()
        result.success(true)
      }

      else -> {
        result.notImplemented()
      }
    }
  }

  private fun isTaskRoot(activity: Activity): Boolean {
    if (activity.isTaskRoot) {
      return true
    }
    return false
  }

}