package com.fireworksdk.bridge.models

import com.fireworksdk.bridge.models.enums.FWCtaButtonTextValue
import org.json.JSONObject

object FWProductInfoViewConfigurationDeserializer {

  private const val CTA_BUTTON = "ctaButton"
  private const val LINK_BUTTON = "linkButton"

  private const val TEXT_KEY = "text"
  private const val BACKGROUND_COLOR_KEY = "backgroundColor"
  private const val TEXT_COLOR_KEY = "textColor"
  private const val FONT_SIZE_KEY = "fontSize"

  private const val IS_HIDDEN_KEY = "isHidden"

  fun deserialize(responseJson: JSONObject?): FWProductInfoViewConfiguration? {
    responseJson ?: return null

    val ctaButtonConfiguration = deserializeCtaButtonConfiguration(responseJson.optJSONObject(CTA_BUTTON))
    val linkButtonConfiguration = deserializeLinkButtonConfiguration(responseJson.optJSONObject(LINK_BUTTON))

    return FWProductInfoViewConfiguration(
      ctaButton = ctaButtonConfiguration,
      linkButton = linkButtonConfiguration,
    )
  }

  private fun deserializeCtaButtonConfiguration(ctaButtonConfiguration: JSONObject?): FWProductInfoViewConfiguration.CtaButtonConfiguration {
    ctaButtonConfiguration ?: return FWProductInfoViewConfiguration.CtaButtonConfiguration()

    val textString = if (ctaButtonConfiguration.has(TEXT_KEY)) ctaButtonConfiguration.optString(TEXT_KEY) else null
    val backgroundColor = if (ctaButtonConfiguration.has(BACKGROUND_COLOR_KEY)) ctaButtonConfiguration.optString(BACKGROUND_COLOR_KEY) else null
    val textColor = if (ctaButtonConfiguration.has(TEXT_COLOR_KEY)) ctaButtonConfiguration.optString(TEXT_COLOR_KEY) else null
    val fontSize = if (ctaButtonConfiguration.has(FONT_SIZE_KEY)) ctaButtonConfiguration.optInt(FONT_SIZE_KEY) else null

    return FWProductInfoViewConfiguration.CtaButtonConfiguration(
      text = FWCtaButtonTextValue.deserialize(textString ?: FWCtaButtonTextValue.AddToCart.rawValue),
      backgroundColor = backgroundColor,
      textColor = textColor,
      fontSize = fontSize,
    )
  }

  private fun deserializeLinkButtonConfiguration(linkButtonConfigJson: JSONObject?): FWProductInfoViewConfiguration.LinkButtonConfiguration {
    linkButtonConfigJson ?: return FWProductInfoViewConfiguration.LinkButtonConfiguration()

    val isHidden = if (linkButtonConfigJson.has(IS_HIDDEN_KEY)) linkButtonConfigJson.optBoolean(IS_HIDDEN_KEY) else null

    return FWProductInfoViewConfiguration.LinkButtonConfiguration(
      isHidden = isHidden,
    )
  }

}
