package com.fireworksdk.bridge.models

import com.fireworksdk.bridge.models.enums.FWCtaButtonTextValue
import org.json.JSONObject

object FWProductInfoViewConfigurationSerializer {

  private const val CTA_BUTTON = "ctaButton"
  private const val LINK_BUTTON = "linkButton"

  private const val TEXT_KEY = "text"
  private const val BACKGROUND_COLOR_KEY = "backgroundColor"
  private const val TEXT_COLOR_KEY = "textColor"
  private const val FONT_SIZE_KEY = "fontSize"

  private const val IS_HIDDEN_KEY = "isHidden"

  fun serialize(model: FWProductInfoViewConfiguration?): JSONObject? {
    model ?: return null
    val jsonObject = JSONObject()
    jsonObject.put(CTA_BUTTON, serializeCtaButtonConfiguration(model.ctaButton))
    jsonObject.put(LINK_BUTTON, serializeLinkButtonConfiguration(model.linkButton))
    return jsonObject
  }

  private fun serializeCtaButtonConfiguration(model: FWProductInfoViewConfiguration.CtaButtonConfiguration?): JSONObject? {
    model ?: return null
    val jsonObject = JSONObject()
    jsonObject.put(TEXT_KEY, FWCtaButtonTextValue.serialize(model.text))
    jsonObject.put(BACKGROUND_COLOR_KEY, model.backgroundColor)
    jsonObject.put(TEXT_COLOR_KEY, model.textColor)
    jsonObject.put(FONT_SIZE_KEY, model.fontSize)
    return jsonObject
  }

  private fun serializeLinkButtonConfiguration(model: FWProductInfoViewConfiguration.LinkButtonConfiguration?): JSONObject? {
    model ?: return null
    val jsonObject = JSONObject()
    jsonObject.put(IS_HIDDEN_KEY, model.isHidden)
    return jsonObject
  }
}