import 'dart:async';

import 'models/method_names.dart';
import 'models/fw_events.dart';
import 'utils/fw_method_channel_util.dart';
import 'utils/model_utils/fw_events_util.dart';
import 'utils/model_utils/method_names_util.dart';

typedef OnLiveStreamEventCallback = FutureOr<void> Function(
    LiveStreamEvent? event);
typedef OnLiveStreamChatEventCallback = FutureOr<void> Function(
    LiveStreamChatEvent? event);

/// The entry class of live stream.
class LiveStream {
  static LiveStream? _instance;

  /// The callback of live stream event.
  OnLiveStreamEventCallback? onLiveStreamEvent;

  /// The callback of live stream chat event.
  OnLiveStreamChatEventCallback? onLiveStreamChatEvent;

  static LiveStream _getInstance() {
    _instance ??= LiveStream._();
    return _instance!;
  }

  LiveStream._() {
    FWMethodChannelUtil.getLiveStreamMethodChannel()
        .setMethodCallHandler((call) async {
      final fwMethod = LiveStreamMethodNameUtil.createMethodName(call.method);
      if (fwMethod == null) {
        return null;
      }

      switch (fwMethod) {
        case LiveStreamMethodName.onLiveStreamEvent:
          final liveStreamEvent = LiveStreamEventUtil.fromJson(call.arguments);
          if (onLiveStreamEvent != null) {
            onLiveStreamEvent!(liveStreamEvent);
          }
          break;
        case LiveStreamMethodName.onLiveStreamChatEvent:
          final liveStreamChatEvent =
              LiveStreamChatEventUtil.fromJson(call.arguments);
          if (onLiveStreamChatEvent != null) {
            onLiveStreamChatEvent!(liveStreamChatEvent);
          }
          break;
      }
      return null;
    });
  }

  factory LiveStream.getInstance() => _getInstance();
}
