enum IOSSystemFontStyle {
  normal,
  italic,
}

enum IOSSystemFontWeight {
  ultraLight,
  thin,
  light,
  regular,
  medium,
  semibold,
  bold,
  heavy,
  black,
}

class IOSFontInfo {
  /// The iOS font name, such as "Helvetica", "Helvetica-Bold".
  /// Only supported on iOS.
  final String? fontName;

  /// The iOS system font style.
  /// The property is ignored when fontName is non-empty.
  /// Only supported on iOS.
  final IOSSystemFontStyle? systemFontStyle;

  /// The iOS system font weight.
  /// The property is ignored when fontName is non-empty or systemFontStyle is italic.
  /// Only supported on iOS.
  final IOSSystemFontWeight? systemFontWeight;
  IOSFontInfo({
    this.fontName,
    this.systemFontStyle,
    this.systemFontWeight,
  });

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) {
      return true;
    }

    if (other is IOSFontInfo) {
      return fontName == other.fontName &&
          systemFontStyle == other.systemFontStyle &&
          systemFontWeight == other.systemFontWeight;
    }

    return false;
  }

  @override
  int get hashCode =>
      fontName.hashCode ^ systemFontStyle.hashCode ^ systemFontWeight.hashCode;

  IOSFontInfo deepCopy() => IOSFontInfo(
        fontName: fontName,
        systemFontStyle: systemFontStyle,
        systemFontWeight: systemFontWeight,
      );
}
