import 'ios_font_info.dart';

class ProductInfoViewConfiguration {
  /// Configuration of shopping CTA button. Only supported on iOS.
  ShoppingCTAButtonConfiguration? ctaButton;

  /// Configuration of link button next to shopping CTA button.
  LinkButtonConfiguration? linkButton;
  ProductInfoViewConfiguration({
    this.ctaButton,
    this.linkButton,
  });

  ProductInfoViewConfiguration deepCopy() => ProductInfoViewConfiguration(
        ctaButton: ctaButton?.deepCopy(),
        linkButton: linkButton?.deepCopy(),
      );
}

enum ShoppingCTAButtonText {
  addToCart,
  shopNow,
}

class ShoppingCTAButtonConfiguration {
  /// The text of shopping CTA button.
  ShoppingCTAButtonText? text;

  /// The background color of CTA button.
  String? backgroundColor;

  /// The text color of CTA button.
  String? textColor;

  /// The font size of CTA button.
  double? fontSize;

  /// The iOS font info of "Add to cart" button.
  /// The property is ignored when fontSize is not set.
  /// Only supported on iOS.
  IOSFontInfo? iOSFontInfo;

  ShoppingCTAButtonConfiguration({
    this.text,
    this.backgroundColor,
    this.textColor,
    this.fontSize,
    this.iOSFontInfo,
  });

  ShoppingCTAButtonConfiguration deepCopy() => ShoppingCTAButtonConfiguration(
        text: text,
        backgroundColor: backgroundColor,
        textColor: textColor,
        fontSize: fontSize,
        iOSFontInfo: iOSFontInfo?.deepCopy(),
      );
}

class LinkButtonConfiguration {
  bool? isHidden;
  LinkButtonConfiguration({
    this.isHidden,
  });

  LinkButtonConfiguration deepCopy() => LinkButtonConfiguration(
        isHidden: isHidden,
      );
}
