import '../../models/video_player_cta_delay.dart';
import '../../models/video_player_cta_style.dart';
import 'ios_font_info_util.dart';

import '../../models/video_player_configuration.dart';

class VideoPlayerConfigurationUtil {
  static Map<String, dynamic> _convertCTAStyleToJson(
      VideoPlayerCTAStyle ctaStyle) {
    Map<String, dynamic> json = {};
    if (ctaStyle.backgroundColor != null &&
        ctaStyle.backgroundColor!.isNotEmpty) {
      json["backgroundColor"] = ctaStyle.backgroundColor!;
    }

    if (ctaStyle.textColor != null && ctaStyle.textColor!.isNotEmpty) {
      json["textColor"] = ctaStyle.textColor!;
    }

    if (ctaStyle.fontSize != null) {
      json["fontSize"] = ctaStyle.fontSize!.toDouble();
    }

    if (ctaStyle.iOSFontInfo != null) {
      json["iOSFontInfo"] = IOSFontInfoUtil.toJson(ctaStyle.iOSFontInfo!);
    }

    return json;
  }

  static Map<String, dynamic> _convertCTADelayToJson(
      VideoPlayerCTADelay ctaDelay) {
    Map<String, dynamic> json = {};
    json["type"] = ctaDelay.type.name;
    json["value"] = ctaDelay.value;

    return json;
  }

  static Map<String, dynamic> toJson(VideoPlayerConfiguration? config) {
    Map<String, dynamic> json = {};

    if (config == null) {
      return json;
    }

    if (config.playerStyle != null) {
      json["playerStyle"] = config.playerStyle!.name;
    }

    if (config.videoCompleteAction != null) {
      json["videoCompleteAction"] = config.videoCompleteAction!.name;
    }

    if (config.showShareButton != null) {
      json["showShareButton"] = config.showShareButton!;
    }

    if (config.ctaButtonStyle != null) {
      json["ctaButtonStyle"] = _convertCTAStyleToJson(config.ctaButtonStyle!);
    }

    if (config.showPlaybackButton != null) {
      json["showPlaybackButton"] = config.showPlaybackButton!;
    }

    if (config.showMuteButton != null) {
      json["showMuteButton"] = config.showMuteButton!;
    }

    if (config.showBranding != null) {
      json["showBranding"] = config.showBranding!;
    }

    if (config.ctaDelay != null) {
      json["ctaDelay"] = _convertCTADelayToJson(config.ctaDelay!);
    }

    if (config.ctaHighlightDelay != null) {
      json["ctaHighlightDelay"] =
          _convertCTADelayToJson(config.ctaHighlightDelay!);
    }

    if (config.shareBaseURL != null) {
      json["shareBaseURL"] = config.shareBaseURL!;
    }

    if (config.ctaWidth != null) {
      json["ctaWidth"] = config.ctaWidth!.name;
    }

    return json;
  }
}
