import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import '../../models/method_names.dart';
import '../../utils/fw_method_channel_util.dart';
import '../../utils/model_utils/fw_error_util.dart';
import '../../utils/model_utils/method_names_util.dart';
import '../../utils/model_utils/video_feed_configuration_util.dart';
import '../../utils/model_utils/video_player_configuration_util.dart';
import 'video_feed_platform_view.dart';

class VideoFeedPlatformHandler {
  final MethodChannel _channel;
  VideoFeedPlatformView _widget;

  VideoFeedPlatformHandler({
    required int viewId,
    required VideoFeedPlatformView widget,
  })  : _channel = FWMethodChannelUtil.getVideoFeedChannel(viewId),
        _widget = widget {
    _channel.setMethodCallHandler(_handleMethodCall);
  }

  void updateWidget(VideoFeedPlatformView widget) {
    final newVideoFeedConfigurationJson =
        VideoFeedConfigurationUtil.toJson(widget.videoFeedConfiguration);

    if (_widget.videoFeedConfiguration != widget.videoFeedConfiguration) {
      _setVideoFeedConfiguration(newVideoFeedConfigurationJson);
    }

    final newVideoPlayerConfigurationJson =
        VideoPlayerConfigurationUtil.toJson(widget.videoPlayerConfiguration);

    if (_widget.videoPlayerConfiguration != widget.videoPlayerConfiguration) {
      _setVideoPlayerConfiguration(newVideoPlayerConfigurationJson);
    }

    _widget = widget;
  }

  void _setVideoFeedConfiguration(Map<String, dynamic> feedConfigJson) {
    final nativeMethod =
        VideoFeedNativeMethodName.setVideoFeedConfiguration.name;

    _channel.invokeMethod(
      nativeMethod,
      feedConfigJson,
    );
  }

  void _setVideoPlayerConfiguration(Map<String, dynamic> playerConfigJson) {
    final nativeMethod =
        VideoFeedNativeMethodName.setVideoPlayerConfiguration.name;

    _channel.invokeMethod(
      nativeMethod,
      playerConfigJson,
    );
  }

  void refresh() {
    final nativeMethod = VideoFeedNativeMethodName.refresh.name;

    _channel.invokeMethod(
      nativeMethod,
    );
  }

  Future<void> setMarginRight(double marginRight) async {
    if (defaultTargetPlatform == TargetPlatform.android) {
      final nativeMethod = VideoFeedNativeMethodName.setMarginRight.name;
      await _channel.invokeMethod(
        nativeMethod,
        marginRight,
      );
    }
  }

  Future<dynamic> _handleMethodCall(MethodCall call) async {
    final methodName = VideoFeedMethodNameUtil.createMethodName(call.method);
    if (methodName == null) {
      return null;
    }

    switch (methodName) {
      case VideoFeedMethodName.onVideoFeedLoadFinished:
        final error = FWErrorUtil.fromJson(call.arguments);
        _widget.onVideoFeedPlatformViewLoadFinished?.call(error);
        break;
    }

    return null;
  }
}
