import 'package:flutter_test/flutter_test.dart';
import 'package:fw_flutter_sdk/src/models/ios_font_info.dart';
import 'package:fw_flutter_sdk/src/models/product_info_view_configuration.dart';
import 'package:fw_flutter_sdk/src/utils/model_utils/product_info_view_configuration_util.dart';

void main() {
  test('ProductInfoViewConfigurationUtil toJson', () {
    final configuration = ProductInfoViewConfiguration(
      ctaButton: ShoppingCTAButtonConfiguration(
        text: ShoppingCTAButtonText.shopNow,
        backgroundColor: "#c0c0c0",
        fontSize: 16,
        textColor: "#000000",
        iOSFontInfo: IOSFontInfo(
          fontName: "Helvetica",
          systemFontStyle: IOSSystemFontStyle.italic,
          systemFontWeight: IOSSystemFontWeight.heavy,
        ),
      ),
      linkButton: LinkButtonConfiguration(isHidden: true),
    );

    final json = ProductInfoViewConfigurationUtil.toJson(configuration);
    final ctaButtonJson = json["ctaButton"];
    final linkButtonJson = json["linkButton"];

    expect(ctaButtonJson is Map<String, dynamic>, true);
    expect(ctaButtonJson["text"], "shopNow");
    expect(ctaButtonJson["backgroundColor"],
        configuration.ctaButton?.backgroundColor);
    expect(ctaButtonJson["fontSize"], configuration.ctaButton?.fontSize);
    expect(ctaButtonJson["textColor"], configuration.ctaButton?.textColor);
    expect(ctaButtonJson["iOSFontInfo"]["fontName"],
        configuration.ctaButton?.iOSFontInfo?.fontName);
    expect(ctaButtonJson["iOSFontInfo"]["systemFontStyle"],
        configuration.ctaButton?.iOSFontInfo?.systemFontStyle?.name);
    expect(ctaButtonJson["iOSFontInfo"]["systemFontWeight"],
        configuration.ctaButton?.iOSFontInfo?.systemFontWeight?.name);
    expect(linkButtonJson is Map<String, dynamic>, true);
    expect(linkButtonJson["isHidden"], configuration.linkButton?.isHidden);
  });
}
