import 'package:flutter_test/flutter_test.dart';
import 'package:fw_flutter_sdk/src/models/product.dart';
import 'package:fw_flutter_sdk/src/utils/model_utils/product_util.dart';

void main() {
  test('ProductUtil toJson', () {
    final units = <ProductUnit>[
      ProductUnit(
        unitId: "test_unit_id1",
        name: "test_unit_name1",
        url: "test_unit_url1",
        imageUrl: "test_unit_imageUrl1",
        price: ProductPrice(
          amount: 100,
          currencyCode: "test_currency_code",
        ),
        isAvailable: true,
        options: <ProductUnitOption>[
          ProductUnitOption(
              name: "test_option_name1", value: "test_option_value2"),
        ],
      ),
    ];
    final product = Product(
      productId: "test_product_id",
      name: "test_product_name",
      description: "test_product_description",
      isAvailable: true,
      units: units,
    );
    final json = ProductUtil.toJson(product);
    expect(json["productId"], product.productId);
    expect(json["name"], product.name);
    expect(json["description"], product.description);
    expect(json["isAvailable"], product.isAvailable);
    expect(json["units"] is List<Map<String, dynamic>>, true);
    final unitsJson = json["units"] as List<Map<String, dynamic>>;
    expect(unitsJson.isNotEmpty, true);
    expect(unitsJson.first["unitId"], product.units?.first.unitId);
    expect(unitsJson.first["name"], product.units?.first.name);
    expect(unitsJson.first["url"], product.units?.first.url);
    expect(unitsJson.first["imageUrl"], product.units?.first.imageUrl);
    expect(unitsJson.first["price"] is Map<String, dynamic>, true);
    final priceJson = unitsJson.first["price"] as Map<String, dynamic>;
    expect(priceJson["amount"], product.units?.first.price?.amount);
    expect(priceJson["currencyCode"], product.units?.first.price?.currencyCode);
    expect(unitsJson.first["isAvailable"], product.units?.first.isAvailable);
    expect(unitsJson.first["options"] is List<Map<String, dynamic>>, true);
    final optionsJson =
        unitsJson.first["options"] as List<Map<String, dynamic>>;
    expect(optionsJson.first["name"], product.units?.first.options?.first.name);
    expect(
        optionsJson.first["value"], product.units?.first.options?.first.value);
  });
}
