import 'package:flutter_test/flutter_test.dart';
import 'package:fw_flutter_sdk/src/models/ios_font_info.dart';
import 'package:fw_flutter_sdk/src/models/video_player_complete_action.dart';
import 'package:fw_flutter_sdk/src/models/video_player_configuration.dart';
import 'package:fw_flutter_sdk/src/models/video_player_cta_delay.dart';
import 'package:fw_flutter_sdk/src/models/video_player_cta_style.dart';
import 'package:fw_flutter_sdk/src/models/video_player_cta_width.dart';
import 'package:fw_flutter_sdk/src/models/video_player_style.dart';
import 'package:fw_flutter_sdk/src/utils/model_utils/video_player_configuration_util.dart';

void main() {
  test('VideoPlayerConfigurationUtil toJson', () {
    final configuration = VideoPlayerConfiguration(
      playerStyle: VideoPlayerStyle.full,
      videoCompleteAction: VideoPlayerCompleteAction.advanceToNext,
      showShareButton: true,
      ctaButtonStyle: VideoPlayerCTAStyle(
        backgroundColor: "#c0c0c0",
        textColor: "#000000",
        fontSize: 16,
        iOSFontInfo: IOSFontInfo(
          fontName: "Helvetica",
          systemFontStyle: IOSSystemFontStyle.italic,
          systemFontWeight: IOSSystemFontWeight.heavy,
        ),
      ),
      showPlaybackButton: true,
      showMuteButton: true,
      showBranding: true,
      ctaDelay: VideoPlayerCTADelay(
        type: VideoPlayerCTADelayType.constant,
        value: 3,
      ),
      ctaHighlightDelay: VideoPlayerCTADelay(
        type: VideoPlayerCTADelayType.constant,
        value: 2,
      ),
      shareBaseURL: "test_share_url",
      ctaWidth: VideoPlayerCTAWidth.fullWidth,
    );
    final json = VideoPlayerConfigurationUtil.toJson(configuration);
    expect(json["playerStyle"], configuration.playerStyle?.name);
    expect(
        json["videoCompleteAction"], configuration.videoCompleteAction?.name);
    expect(json["showShareButton"], configuration.showShareButton);
    expect(json["ctaButtonStyle"] is Map<String, dynamic>, true);
    final ctaButtonStyleJson = json["ctaButtonStyle"] as Map<String, dynamic>;
    expect(ctaButtonStyleJson["backgroundColor"],
        configuration.ctaButtonStyle?.backgroundColor);
    expect(ctaButtonStyleJson["textColor"],
        configuration.ctaButtonStyle?.textColor);
    expect(
        ctaButtonStyleJson["fontSize"], configuration.ctaButtonStyle?.fontSize);
    expect(ctaButtonStyleJson["iOSFontInfo"]["fontName"],
        configuration.ctaButtonStyle?.iOSFontInfo?.fontName);
    expect(ctaButtonStyleJson["iOSFontInfo"]["systemFontStyle"],
        configuration.ctaButtonStyle?.iOSFontInfo?.systemFontStyle?.name);
    expect(ctaButtonStyleJson["iOSFontInfo"]["systemFontWeight"],
        configuration.ctaButtonStyle?.iOSFontInfo?.systemFontWeight?.name);
    expect(json["showPlaybackButton"], configuration.showPlaybackButton);
    expect(json["showMuteButton"], configuration.showMuteButton);
    expect(json["showBranding"], configuration.showBranding);

    expect(json["ctaDelay"]["type"], configuration.ctaDelay?.type.name);
    expect(json["ctaDelay"]["value"], configuration.ctaDelay?.value);
    expect(json["ctaHighlightDelay"]["type"],
        configuration.ctaHighlightDelay?.type.name);
    expect(json["ctaHighlightDelay"]["value"],
        configuration.ctaHighlightDelay?.value);
    expect(json["shareBaseURL"], configuration.shareBaseURL);
    expect(json["ctaWidth"], configuration.ctaWidth?.name);
  });
}
